/*Ŀ
                                                                          
   LIFE.C v1.0                                                            
                                                                          
   A graphical representation of Conway's famous Game of Life algorithm.  
                                                                          
   All functions, logic, and code by Justin Powell.                       
                                                                          
   2/13/1997                                                              
                                                                          
  */
/*Ŀ
   PREPROCESSORS                                                          
  */

    #include<stdlib.h>
    #include<math.h>
    #include<stdio.h>
    #include<time.h>

// PSI -- linux include
    #include <unistd.h>
    #include <vga.h>          
    #include <vgakeyboard.h>
    #include <ncurses.h>
    
// PSI -- color defines
    #define COLOR1 1 		// Dark Blue
    #define COLOR2 10		// Green
    #define COLOR3 12		// Red
    #define COLOR4 15		// White
    

/*Ŀ
   MACRO DEFINITIONS                                                      
  */

// PSI -- Changed from original dos defs

    #define MCGA_MODE G320x200x256 /* BIOS mode value for initialize_mode(); */
    #define TEXT_MODE TEXT 	   /* BIOS mode value for initialize_mode(); */

/*Ŀ
   GLOBAL VARIABLES                                                       
  */

    int world[107][67];
    int neighbors[107][67];
    int XIN1; /* high number less snails upon generation */

/*Ŀ
   GRAPHICS FUNCTIONS                                                     
  */

    void initialize_mode(int video_mode);
    void cls(void);
    void set_dac(int dac,char r,char g,char b);
    void set_palette(char palette[768]);
    void put_pixel(int x,int y,unsigned char color);
    void wait_retrace(void);

    #include"life_gfx.c"
    #include"linux_port.c"

    void initialize_world(void);
    void draw_square(int x,int y,char color);
    void show_world(void);
    void update_neighbors(void);
    void update_world(int nuke);
    int  neighbor_plague(int x,int y);
    void nuke_site(int explosion_size);

/*Ŀ
   CODE                                                                   
  */

    void initialize_world(void)
    {
        int x,y;

        randomize();
        for(y=0;y<67;y++)
        {
            for(x=0;x<107;x++)
            {
                if(gimmerand(XIN1)==1) world[x][y]=1;
                else world[x][y]=0;
            }
        }
    }

    void draw_square(int x,int y,char color)
    {
        int i,j;

        for(i=y;i<(y+2);i++)
        {
            for(j=x;j<(x+2);j++)
            {
                put_pixel(j,i,color);
            }
        }
    }

    void show_world(void)
    {
        int i,j;

        for(i=0;i<107;i++)
        {
            for(j=0;j<67;j++)
            {
                if(world[i][j]==0) draw_square((i*3),(j*3),COLOR1);
                if(world[i][j]==1) draw_square((i*3),(j*3),COLOR2);
                if(world[i][j]==2) draw_square((i*3),(j*3),COLOR3);
		// PSI - new nuke
                if(world[i][j]==4) draw_square((i*3),(j*3),COLOR4);
            }
        }
    }

    void update_neighbors(void)
    {
        int x,y;

        for(y=0;y<67;y++)
        {
            for(x=0;x<107;x++)
            {
                neighbors[x][y]=0;
            }
        }
        for(y=0;y<67;y++)
        {
            for(x=0;x<107;x++)
            {
                if(((x-1)>0)&&((y-1)>0)) neighbors[x][y]+=world[x-1][y-1];
                if((y-1)>0) neighbors[x][y]+=world[x][y-1];
                if(((x+1)<107)&&((y-1)>0))neighbors[x][y]+=world[x+1][y-1];
                if((x-1)>0) neighbors[x][y]+=world[x-1][y];
                if((x+1)<107) neighbors[x][y]+=world[x+1][y];
                if(((x-1)>0)&&((y+1)<67)) neighbors[x][y]+=world[x-1][y+1];
                if((y+1)<67) neighbors[x][y]+=world[x][y+1];
                if(((x+1)<107)&&((y+1)<67)) neighbors[x][y]+=world[x+1][y+1];
            }
        }
    }

    void update_world(int nuke)
    {
        int x,y;

        for(y=0;y<67;y++)
        {
            for(x=0;x<107;x++)
            {
		// PSI -- work with nukes
		if((nuke==3)&&(world[x][y]==4)) world[x][y]=0;

                if(world[x][y]==2)
                {
                    if(gimmerand(20)==1) world[x][y]=0;
                    //1 in 20 chance of dying from the plague
                }
                if((world[x][y])&&(neighbor_plague(x,y)))
                {
                    if((gimmerand(2)==1)||(gimmerand(3)==1)) world[x][y]=2;
                    //1 in 1.5 chance of catching the plague
                    //catches plague from neighbor
                }
                if((world[x][y]==1)&&(neighbors[x][y]<2))
                {
                    world[x][y]=0; //dies of lonliness
                }
                if((world[x][y]==1)&&(neighbors[x][y]>4))
                {
                    world[x][y]=0; //dies of overcrowding
                }
                if((world[x][y]==1)&&(neighbors[x][y]==2))
                {
                    if(!neighbor_plague(x,y)) world[x][y]=1;
                    //survives if both neighbors are healthy
                }
                if((world[x][y]==0)&&(neighbors[x][y]==3))
                {
                    if(!neighbor_plague(x,y)) world[x][y]=1;
                    //springs to life if all neighbors are healthy
                }
            }
        }
    }

    int neighbor_plague(int x,int y)
    {
        if(((x-1)>0)&&((y-1)>0)) if(world[x-1][y-1]==2) return(1);
        if((y-1)>0) if(world[x][y-1]==2) return(1);
        if(((x+1)<107)&&((y-1)>0)) if(world[x+1][y-1]==2) return(1);
        if((x-1)>0) if(world[x-1][y]==2) return(1);
        if((x+1)<107) if(world[x+1][y]==2) return(1);
        if(((x-1)>0)&&((y+1)<67)) if(world[x-1][y+1]==2) return(1);
        if((y+1)<67) if(world[x][y+1]==2) return(1);
        if(((x+1)<107)&&((y+1)<67)) if(world[x+1][y+1]==2) return(1);
        return(0);
    }


    void nuke_site(int explosion_size)
    {
        int hitx;
        int hity;
        int i,j;

        hitx=gimmerand(107);
        hity=gimmerand(67);
        for(i=hity;i<hity+explosion_size;i++)
        {
            for(j=hitx;j<hitx+explosion_size;j++)
            {
                if((i>0)&&(i<67)&&(j>0)&&(j<107)) world[i][j]=4; //PSI
            }
        }
    }

/*Ŀ
   main(void)                                                             
  Ĵ
   The main program code.                                                 
  */

    void main(int argc,char *argv[])
    {
        int x,y;
        int plague=0,ptoggle=0,command=0;
        int nuke=0,ntoggle=0;
        int speed=0;
	
	WINDOW *screen;

        printf("Ŀ");
        printf(" v1.0                    The Game of Life Simulation                          ");
        printf(" (Linux Port)                                                                 ");
        printf("                                                                              ");
        printf(" This is a representation of mathematician J.H. Conway's Game of Life         ");
        printf(" simulation by Justin Powell, a programmer with too much time on his hands.   ");
        printf("                                                                              ");
        printf(" The game of life uses a two dimentional array of cells to represent the      ");
        printf(" changing world of lifeforms based on population and breeding rules.  They    ");
        printf(" live, exist, or die based on crowding or lonliness conditions around them.   ");
        printf(" For the complete algorithm outline see LIFE.DOC.                             ");
        printf("                                                                              ");
        printf(" If anyone is interested in looking at the source code, drop me some email and");
        printf(" I'd be more than happy to share it, all functions were developed from scratch");
        printf(" by me (graphics functions included), and the whole project took me about 20  ");
        printf(" consecutive hours to code.  I'm at (jpowell@global2000.net).                 ");
        printf(" Linux Port (psionic@darklight.com)                                           ");
        printf("");
        if(argc<2)
        {
            printf("\nYou must enter a number between 2 and 50.\n");
            printf("This value is used to determine the initial percentage of life in the grid.\n");
            printf("A low number means more life, a high number - less.\n");
            exit(0);
        }
        printf("[PAUSE]");
        getchar();
        XIN1=atoi(argv[1]);
        initialize_mode(MCGA_MODE);
        initialize_world();
        screen=initscr();
        show_world();
        while((command!='Q')&&(command!='q'))			// PSI - Put keyboard handler here
        {
            update_neighbors();
            update_world(nuke);
	    if(nuke==2) nuke++;
	    else if(nuke==3) nuke=0;

            show_world();
            if(plague)
            {
                if(ptoggle==0)
                {
                    while(!ptoggle)
                    {
                        x=gimmerand(107);
                        y=gimmerand(97);
                        if(world[x][y]==0)
                        {
                            world[x][y]=2;
                            ptoggle=1;
                        }
                    }
                }
            }
            if(nuke)
            {
                if(ntoggle==0)
                {
                    nuke_site(gimmerand(50)+50); //nuke a random area
                    ntoggle=1;
		    nuke=2;
                }
            }
            
            // PSI -- input handling completely re-written from DOS ver
            if(speed==0) nodelay(screen,TRUE);
            else halfdelay(speed);
            command=getch();
	    switch(command) {
		case 'P':
		case 'p':
		        plague=1;
                	ptoggle=0;
			break;
		case 'N':
		case 'n':
	                nuke=1;
            		ntoggle=0;
			break;
		case '+':
			if(speed<200) speed++;
			break;
		case '-':
			if(speed>0) speed--;
	    	default: break;
	    }

        }

        initialize_mode(TEXT_MODE);
        exit(0);
    }


